"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemporalCluster = void 0;
const constructs_1 = require("constructs");
const aws_ec2_1 = require("aws-cdk-lib/aws-ec2");
const aws_rds_1 = require("aws-cdk-lib/aws-rds");
const aws_ecs_1 = require("aws-cdk-lib/aws-ecs");
const TemporalDatastore_1 = require("./TemporalDatastore");
const TemporalVersion_1 = require("./TemporalVersion");
const aws_cdk_lib_1 = require("aws-cdk-lib");
const aws_efs_1 = require("aws-cdk-lib/aws-efs");
const EfsFileResource_1 = require("./customResources/efsFileResource/EfsFileResource");
const TemporalConfiguration_1 = require("./configurations/TemporalConfiguration");
const aws_servicediscovery_1 = require("aws-cdk-lib/aws-servicediscovery");
const ServerServices_1 = require("./services/ServerServices");
const TemporalDatabaseResource_1 = require("./customResources/temporal/TemporalDatabaseResource");
const lodash_1 = require("lodash");
const defaultMachineProperties = {
    cpu: 256,
    memoryLimitMiB: 512,
    cpuArchitecture: aws_ecs_1.CpuArchitecture.X86_64,
};
class TemporalCluster extends constructs_1.Construct {
    constructor(scope, id, clusterProps) {
        super(scope, id);
        this.name = clusterProps.clusterName ?? aws_cdk_lib_1.Names.uniqueId(this);
        this.temporalVersion = clusterProps.temporalVersion ?? TemporalVersion_1.TemporalVersion.LATEST;
        this.temporalConfig = new TemporalConfiguration_1.TemporalConfiguration();
        const servicesProps = this.resolveServiceProps(clusterProps.services);
        // FIXME: Add support for mixed datastores configuration (ie. SQL+Cassandra or SQL+ES)
        const datastore = this.getOrCreateDatastore(clusterProps);
        const mainDatabase = new TemporalDatabaseResource_1.TemporalDatabase(this, 'MainDatabase', {
            temporalCluster: this,
            datastore: datastore,
            vpc: clusterProps.vpc,
            databaseName: 'temporal',
            schemaType: 'main',
            removalPolicy: clusterProps.removalPolicy,
        });
        this.temporalConfig.attachDatabase(mainDatabase);
        const visibilityDatabase = new TemporalDatabaseResource_1.TemporalDatabase(this, 'VisibilityDatabase', {
            temporalCluster: this,
            datastore: datastore,
            vpc: clusterProps.vpc,
            databaseName: 'temporal_visibility',
            schemaType: 'visibility',
            removalPolicy: clusterProps.removalPolicy,
        });
        this.temporalConfig.attachDatabase(visibilityDatabase);
        this.ecsCluster = this.getOrCreateEcsCluster(clusterProps);
        this.configEfs = this.setupConfigFileSystem(clusterProps, this.temporalConfig);
        this.services = {
            // FIXME: Replace the auto-setup image by CustomResources that only runs setup tasks
            // autoSetup: new AutoSetupService(this, { machine: servicesProps.autoSetup.machine }),
            frontend: new ServerServices_1.FrontendService(this, { machine: servicesProps.frontend.machine }),
            matching: new ServerServices_1.MatchingService(this, { machine: servicesProps.matching.machine }),
            history: new ServerServices_1.HistoryService(this, { machine: servicesProps.history.machine }),
            worker: new ServerServices_1.WorkerService(this, { machine: servicesProps.worker.machine }),
            web: servicesProps.web.enabled ? new ServerServices_1.WebService(this, { machine: servicesProps.web.machine }) : undefined,
        };
        if (clusterProps.cloudMapRegistration) {
            this.services.frontend.fargateService.enableCloudMap({
                name: clusterProps.cloudMapRegistration?.serviceName,
                cloudMapNamespace: clusterProps.cloudMapRegistration?.namespace,
                dnsRecordType: aws_servicediscovery_1.DnsRecordType.A,
            });
        }
        this.wireUpNetworkAuthorizations([mainDatabase, visibilityDatabase]);
    }
    // FIXME: Refactor this
    resolveServiceProps(services) {
        var _a;
        const out = {};
        for (const service of ['frontend', 'history', 'matching', 'worker', 'autoSetup', 'web']) {
            out[service] = {
                ...services?.defaults,
                ...services?.[service],
                machine: {
                    ...defaultMachineProperties,
                    ...services?.defaults?.machine,
                    ...services?.[service]?.machine,
                },
            };
        }
        (_a = out.web).enabled ?? (_a.enabled = true);
        return out;
    }
    getOrCreateDatastore(props) {
        // FIXME: Validate that requested/provided datastore is supported by requested temporal server version
        // https://docs.temporal.io/docs/server/versions-and-dependencies/#server-versioning-and-support-policy
        if (props.datastore && props.datastoreOptions) {
            throw `You must specify either a datastore or datastoreOptions, not both.`;
        }
        else if (props.datastore) {
            return props.datastore;
        }
        else {
            return new TemporalDatastore_1.AuroraServerlessTemporalDatastore(this, 'Datastore', {
                engine: aws_rds_1.DatabaseClusterEngine.auroraMysql({ version: aws_rds_1.AuroraMysqlEngineVersion.VER_2_10_1 }),
                vpc: props.vpc,
                removalPolicy: props.removalPolicy,
                ...props.datastoreOptions,
            });
        }
    }
    getOrCreateEcsCluster(props) {
        if (props.ecsCluster) {
            return props.ecsCluster;
        }
        else {
            return new aws_ecs_1.Cluster(this, 'EcsCluster', {
                vpc: props.vpc,
                enableFargateCapacityProviders: true,
                containerInsights: true,
            });
        }
    }
    setupConfigFileSystem(props, temporalConfig) {
        const configEfs = new aws_efs_1.FileSystem(this, 'ConfigFS', {
            vpc: props.vpc,
            removalPolicy: props.removalPolicy,
        });
        // Temporal server image builds the main configuration file, from a template file
        // Ideally, we would replace that template file with something that already contains
        // most of our configuration items. However, we faced issues implementing that:
        // posix permissions when the script tries to write new dockerized config file; ECS doesn't
        // allow to map a single file from an EFS volume (only a directory)...
        // At present, we rely on the default template file, and pass all config items through env variables.
        // A possible alternative: https://kichik.com/2020/09/10/mounting-configuration-files-in-fargate/
        // new EfsFile(this, 'TemporalConfig', {
        //     fileSystem: configEfs,
        //     path: `/temporal/config/config_template.yaml`,
        //     vpc: props.vpc,
        //     contents: Lazy.string({ produce: () => temporalConfig.stringifyConfiguration() }),
        // });
        new EfsFileResource_1.EfsFile(this, 'TemporalDynamicConfig', {
            fileSystem: configEfs,
            path: `/temporal/dynamic_config/dynamic_config.yaml`,
            vpc: props.vpc,
            contents: aws_cdk_lib_1.Lazy.string({ produce: () => temporalConfig.stringifyDynamic() }),
        });
        new EfsFileResource_1.EfsFile(this, 'TemporalWebConfig', {
            fileSystem: configEfs,
            path: `/temporal/web_config/web_config.yaml`,
            vpc: props.vpc,
            contents: aws_cdk_lib_1.Lazy.string({ produce: () => temporalConfig.stringifyWeb() }),
        });
        return configEfs;
    }
    wireUpNetworkAuthorizations(databases) {
        // Wire up network authorizations
        // FIXME: Refactor all of this
        // FIXME: At this time, the autoSetup plays all roles; it makes wiring up significantly more complicated...
        function allow(sources, port, targets) {
            for (const target of targets)
                for (const source of sources) {
                    target.fargateService.connections.allowFrom(source.fargateService, aws_ec2_1.Port.tcp(port));
                }
        }
        // FIXME: Not everyone might want this...
        this.services.frontend.fargateService.connections.allowFromAnyIpv4(aws_ec2_1.Port.tcp(7233));
        // this.services.autoSetup.fargateService.connections.allowFromAnyIpv4(Port.tcp(7233)); // FIXME: Remove
        const frontendNodes = [this.services.frontend /*, this.services.autoSetup*/];
        const historyNodes = [this.services.history /*, this.services.autoSetup*/];
        const matchingNodes = [this.services.matching /*, this.services.autoSetup*/];
        const workerNodes = [this.services.worker /*, this.services.autoSetup*/];
        allow(frontendNodes, 7234, historyNodes);
        allow(frontendNodes, 7235, matchingNodes);
        allow(frontendNodes, 7239, workerNodes);
        allow(historyNodes, 7235, matchingNodes);
        allow(matchingNodes, 7233, frontendNodes);
        allow(matchingNodes, 7234, historyNodes);
        allow(workerNodes, 7233, frontendNodes);
        allow(frontendNodes, 6933, frontendNodes);
        allow(historyNodes, 6934, historyNodes);
        allow(matchingNodes, 6935, matchingNodes);
        allow(workerNodes, 6939, workerNodes);
        const datastores = (0, lodash_1.uniq)(databases.map((db) => db.datastore));
        for (const datastore of datastores) {
            // Grant network access from the fargate service to the RDS datastore
            datastore.connections.allowDefaultPortFrom(this.services.frontend.fargateService);
            datastore.connections.allowDefaultPortFrom(this.services.history.fargateService);
            datastore.connections.allowDefaultPortFrom(this.services.matching.fargateService);
            // datastore.connections.allowDefaultPortFrom(this.services.autoSetup.fargateService);
        }
    }
}
exports.TemporalCluster = TemporalCluster;
//# sourceMappingURL=TemporalCluster.js.map